/*
 * Copyright (C) 2020 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Power.h"

#include <android-base/logging.h>
#include <android/binder_manager.h>
#include <android/binder_process.h>

using aidl::android::hardware::power::impl::linaro::Power;

static int transportCallback(__attribute__((unused)) int fd, __attribute__((unused)) int event,
                             __attribute__((unused)) void *data)
{
	ABinderProcess_handlePolledCommands();
        return 1;
}

int main()
{
	int fd;
	Looper *looper;

	looper = new Looper(false);
	if (!looper)
		return EXIT_FAILURE;

	std::shared_ptr<Power> power = ndk::SharedRefBase::make<Power>(looper);

	ABinderProcess_setThreadPoolMaxThreadCount(0);

	const std::string instance = std::string() + Power::descriptor + "/default";
	binder_status_t status = AServiceManager_addService(power->asBinder().get(), instance.c_str());

	CHECK_EQ(status, STATUS_OK);

	ABinderProcess_setupPolling(&fd);

	looper->addFd(fd, 0, Looper::EVENT_INPUT, transportCallback, NULL);

	looper->pollAll(-1);

	return EXIT_FAILURE;  // should not reach
}
